/*
 * Copyright (C) 2018 Apple Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY APPLE INC. AND ITS CONTRIBUTORS ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL APPLE INC. OR ITS CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 */

#pragma once

#include "Color.h"
#include "FloatSize.h"
#include <wtf/RetainPtr.h>

#if PLATFORM(COCOA)
OBJC_CLASS NSShadow;
#endif

namespace WebCore {

struct FontShadow {
    template<class Encoder> void encode(Encoder&) const;
    template<class Decoder> static bool decode(Decoder&, FontShadow&);

#if PLATFORM(COCOA)
    RetainPtr<NSShadow> createShadow() const;
#endif

    Color color;
    FloatSize offset;
    double blurRadius { 0 };
};

template<class Encoder>
void FontShadow::encode(Encoder& encoder) const
{
    encoder << color << offset << blurRadius;
}

template<class Decoder>
bool FontShadow::decode(Decoder& decoder, FontShadow& shadow)
{
    if (!decoder.decode(shadow.color))
        return false;

    if (!decoder.decode(shadow.offset))
        return false;

    if (!decoder.decode(shadow.blurRadius))
        return false;

    return true;
}

} // namespace WebCore
