/*
 * Copyright (C) 2013 Google Inc. All rights reserved.
 * Copyright (C) 2013-2016 Apple Inc. All rights reserved.
 * Copyright (C) 2014 University of Washington. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY APPLE INC. AND ITS CONTRIBUTORS ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL APPLE INC. OR ITS CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 */

// DO NOT EDIT THIS FILE. It is automatically generated from Automation.json
// by the script: Source/JavaScriptCore/inspector/scripts/generate-inspector-protocol-bindings.py

#pragma once

#include <JavaScriptCore/InspectorProtocolTypes.h>
#include <wtf/Assertions.h>

namespace Inspector {



namespace Protocol {



// Forward declarations.
namespace Automation {
class Point;
class Size;
class Rect;
class BrowsingContext;
class KeyboardInteraction;
class Cookie;
class SessionPermissionData;
class InputSource;
class InteractionStep;
class InputSourceState;
enum class CoordinateSystem;
enum class BrowsingContextPresentation;
enum class ErrorMessage;
enum class PageLoadStrategy;
enum class MouseInteraction;
enum class MouseButton;
enum class KeyModifier;
enum class VirtualKey;
enum class KeyboardInteractionType;
enum class SessionPermission;
enum class InputSourceType;
enum class MouseMoveOrigin;
} // Automation
// End of forward declarations.


// Typedefs.
namespace Automation {
/* An opaque identifier for a browsing context. */
typedef String BrowsingContextHandle;
/* An opaque identifier for a frame in a page. */
typedef String FrameHandle;
/* An opaque identifier for a node in a page. */
typedef String NodeHandle;
} // Automation
// End of typedefs.

namespace AutomationHelpers {

String getEnumConstantValue(int code);

template<typename T> String getEnumConstantValue(T enumValue)
{
    return getEnumConstantValue(static_cast<int>(enumValue));
}

} // namespace AutomationHelpers

namespace Automation {
class Point : public JSON::ObjectBase {
public:
    enum {
        NoFieldsSet = 0,
        XSet = 1 << 0,
        YSet = 1 << 1,
        AllFieldsSet = (XSet | YSet)
    };

    template<int STATE>
    class Builder {
    private:
        RefPtr<JSON::Object> m_result;

        template<int STEP> Builder<STATE | STEP>& castState()
        {
            return *reinterpret_cast<Builder<STATE | STEP>*>(this);
        }

        Builder(Ref</*Point*/JSON::Object>&& object)
            : m_result(WTFMove(object))
        {
            COMPILE_ASSERT(STATE == NoFieldsSet, builder_created_in_non_init_state);
        }
        friend class Point;
    public:

        Builder<STATE | XSet>& setX(double value)
        {
            COMPILE_ASSERT(!(STATE & XSet), property_x_already_set);
            m_result->setDouble("x"_s, value);
            return castState<XSet>();
        }

        Builder<STATE | YSet>& setY(double value)
        {
            COMPILE_ASSERT(!(STATE & YSet), property_y_already_set);
            m_result->setDouble("y"_s, value);
            return castState<YSet>();
        }

        Ref<Point> release()
        {
            COMPILE_ASSERT(STATE == AllFieldsSet, result_is_not_ready);
            COMPILE_ASSERT(sizeof(Point) == sizeof(JSON::Object), cannot_cast);

            Ref<JSON::Object> jsonResult = m_result.releaseNonNull();
            auto result = WTFMove(*reinterpret_cast<Ref<Point>*>(&jsonResult));
            return result;
        }
    };

    /*
     * Synthetic constructor:
     * Ref<Point> result = Point::create()
     *     .setX(...)
     *     .setY(...)
     *     .release();
     */
    static Builder<NoFieldsSet> create()
    {
        return Builder<NoFieldsSet>(JSON::Object::create());
    }
};

class Size : public JSON::ObjectBase {
public:
    enum {
        NoFieldsSet = 0,
        WidthSet = 1 << 0,
        HeightSet = 1 << 1,
        AllFieldsSet = (WidthSet | HeightSet)
    };

    template<int STATE>
    class Builder {
    private:
        RefPtr<JSON::Object> m_result;

        template<int STEP> Builder<STATE | STEP>& castState()
        {
            return *reinterpret_cast<Builder<STATE | STEP>*>(this);
        }

        Builder(Ref</*Size*/JSON::Object>&& object)
            : m_result(WTFMove(object))
        {
            COMPILE_ASSERT(STATE == NoFieldsSet, builder_created_in_non_init_state);
        }
        friend class Size;
    public:

        Builder<STATE | WidthSet>& setWidth(double value)
        {
            COMPILE_ASSERT(!(STATE & WidthSet), property_width_already_set);
            m_result->setDouble("width"_s, value);
            return castState<WidthSet>();
        }

        Builder<STATE | HeightSet>& setHeight(double value)
        {
            COMPILE_ASSERT(!(STATE & HeightSet), property_height_already_set);
            m_result->setDouble("height"_s, value);
            return castState<HeightSet>();
        }

        Ref<Size> release()
        {
            COMPILE_ASSERT(STATE == AllFieldsSet, result_is_not_ready);
            COMPILE_ASSERT(sizeof(Size) == sizeof(JSON::Object), cannot_cast);

            Ref<JSON::Object> jsonResult = m_result.releaseNonNull();
            auto result = WTFMove(*reinterpret_cast<Ref<Size>*>(&jsonResult));
            return result;
        }
    };

    /*
     * Synthetic constructor:
     * Ref<Size> result = Size::create()
     *     .setWidth(...)
     *     .setHeight(...)
     *     .release();
     */
    static Builder<NoFieldsSet> create()
    {
        return Builder<NoFieldsSet>(JSON::Object::create());
    }
};

class Rect : public JSON::ObjectBase {
public:
    enum {
        NoFieldsSet = 0,
        OriginSet = 1 << 0,
        SizeSet = 1 << 1,
        AllFieldsSet = (OriginSet | SizeSet)
    };

    template<int STATE>
    class Builder {
    private:
        RefPtr<JSON::Object> m_result;

        template<int STEP> Builder<STATE | STEP>& castState()
        {
            return *reinterpret_cast<Builder<STATE | STEP>*>(this);
        }

        Builder(Ref</*Rect*/JSON::Object>&& object)
            : m_result(WTFMove(object))
        {
            COMPILE_ASSERT(STATE == NoFieldsSet, builder_created_in_non_init_state);
        }
        friend class Rect;
    public:

        Builder<STATE | OriginSet>& setOrigin(RefPtr<Inspector::Protocol::Automation::Point> value)
        {
            COMPILE_ASSERT(!(STATE & OriginSet), property_origin_already_set);
            m_result->setObject("origin"_s, value);
            return castState<OriginSet>();
        }

        Builder<STATE | SizeSet>& setSize(RefPtr<Inspector::Protocol::Automation::Size> value)
        {
            COMPILE_ASSERT(!(STATE & SizeSet), property_size_already_set);
            m_result->setObject("size"_s, value);
            return castState<SizeSet>();
        }

        Ref<Rect> release()
        {
            COMPILE_ASSERT(STATE == AllFieldsSet, result_is_not_ready);
            COMPILE_ASSERT(sizeof(Rect) == sizeof(JSON::Object), cannot_cast);

            Ref<JSON::Object> jsonResult = m_result.releaseNonNull();
            auto result = WTFMove(*reinterpret_cast<Ref<Rect>*>(&jsonResult));
            return result;
        }
    };

    /*
     * Synthetic constructor:
     * Ref<Rect> result = Rect::create()
     *     .setOrigin(...)
     *     .setSize(...)
     *     .release();
     */
    static Builder<NoFieldsSet> create()
    {
        return Builder<NoFieldsSet>(JSON::Object::create());
    }
};

/* The coordinate system in which rects, points, and sizes are to be interpreted. */
enum class CoordinateSystem {
    Page = 0,
    Viewport = 1,
}; // enum class CoordinateSystem
/* How a browsing context is presented by the user agent. A browsing context is considered a tab if it shares an OS-level window with another browsing context. */
enum class BrowsingContextPresentation {
    Window = 2,
    Tab = 3,
}; // enum class BrowsingContextPresentation
/* This enum contains predefined error messages that can be used to signal a well-defined error condition, such as a missing implementation, unknown window handle, and so forth. The backend signals one of these errors by using it as a prefix of the commands's error message (the errorString argument in generated C++ backend dispatchers). This will be reported to the frontend as a protocol error with a JSON-RPC error code of 'ServerError'. It is up to the frontend whether and how to deal with errors. */
enum class ErrorMessage {
    InternalError = 4,
    Timeout = 5,
    JavaScriptError = 6,
    JavaScriptTimeout = 7,
    WindowNotFound = 8,
    FrameNotFound = 9,
    NodeNotFound = 10,
    InvalidNodeIdentifier = 11,
    InvalidElementState = 12,
    NoJavaScriptDialog = 13,
    NotImplemented = 14,
    MissingParameter = 15,
    InvalidParameter = 16,
    InvalidSelector = 17,
    ElementNotInteractable = 18,
    ElementNotSelectable = 19,
    ScreenshotError = 20,
    UnexpectedAlertOpen = 21,
    TargetOutOfBounds = 22,
}; // enum class ErrorMessage
/* Enumerates different page load strategies. */
enum class PageLoadStrategy {
    None = 23,
    Eager = 24,
    Normal = 25,
}; // enum class PageLoadStrategy
/* A handle representing an open window or tab in the automation session. */
class BrowsingContext : public JSON::ObjectBase {
public:
    enum {
        NoFieldsSet = 0,
        HandleSet = 1 << 0,
        ActiveSet = 1 << 1,
        UrlSet = 1 << 2,
        WindowSizeSet = 1 << 3,
        WindowOriginSet = 1 << 4,
        AllFieldsSet = (HandleSet | ActiveSet | UrlSet | WindowSizeSet | WindowOriginSet)
    };

    template<int STATE>
    class Builder {
    private:
        RefPtr<JSON::Object> m_result;

        template<int STEP> Builder<STATE | STEP>& castState()
        {
            return *reinterpret_cast<Builder<STATE | STEP>*>(this);
        }

        Builder(Ref</*BrowsingContext*/JSON::Object>&& object)
            : m_result(WTFMove(object))
        {
            COMPILE_ASSERT(STATE == NoFieldsSet, builder_created_in_non_init_state);
        }
        friend class BrowsingContext;
    public:

        Builder<STATE | HandleSet>& setHandle(const String& value)
        {
            COMPILE_ASSERT(!(STATE & HandleSet), property_handle_already_set);
            m_result->setString("handle"_s, value);
            return castState<HandleSet>();
        }

        Builder<STATE | ActiveSet>& setActive(bool value)
        {
            COMPILE_ASSERT(!(STATE & ActiveSet), property_active_already_set);
            m_result->setBoolean("active"_s, value);
            return castState<ActiveSet>();
        }

        Builder<STATE | UrlSet>& setUrl(const String& value)
        {
            COMPILE_ASSERT(!(STATE & UrlSet), property_url_already_set);
            m_result->setString("url"_s, value);
            return castState<UrlSet>();
        }

        Builder<STATE | WindowSizeSet>& setWindowSize(RefPtr<Inspector::Protocol::Automation::Size> value)
        {
            COMPILE_ASSERT(!(STATE & WindowSizeSet), property_windowSize_already_set);
            m_result->setObject("windowSize"_s, value);
            return castState<WindowSizeSet>();
        }

        Builder<STATE | WindowOriginSet>& setWindowOrigin(RefPtr<Inspector::Protocol::Automation::Point> value)
        {
            COMPILE_ASSERT(!(STATE & WindowOriginSet), property_windowOrigin_already_set);
            m_result->setObject("windowOrigin"_s, value);
            return castState<WindowOriginSet>();
        }

        Ref<BrowsingContext> release()
        {
            COMPILE_ASSERT(STATE == AllFieldsSet, result_is_not_ready);
            COMPILE_ASSERT(sizeof(BrowsingContext) == sizeof(JSON::Object), cannot_cast);

            Ref<JSON::Object> jsonResult = m_result.releaseNonNull();
            auto result = WTFMove(*reinterpret_cast<Ref<BrowsingContext>*>(&jsonResult));
            return result;
        }
    };

    /*
     * Synthetic constructor:
     * Ref<BrowsingContext> result = BrowsingContext::create()
     *     .setHandle(...)
     *     .setActive(...)
     *     .setUrl(...)
     *     .setWindowSize(...)
     *     .setWindowOrigin(...)
     *     .release();
     */
    static Builder<NoFieldsSet> create()
    {
        return Builder<NoFieldsSet>(JSON::Object::create());
    }
};

/* Enumerates different ways of interacting with a mouse cursor. */
enum class MouseInteraction {
    Move = 26,
    Down = 27,
    Up = 28,
    SingleClick = 29,
    DoubleClick = 30,
}; // enum class MouseInteraction
/* Enumerates different mouse buttons that can be used. */
enum class MouseButton {
    None = 23,
    Left = 31,
    Middle = 32,
    Right = 33,
}; // enum class MouseButton
/* Enumerates different key modifiers that can remain pressed during other mouse/touch interactions. */
enum class KeyModifier {
    CapsLock = 34,
    Control = 35,
    Shift = 36,
    Meta = 37,
    Alt = 38,
}; // enum class KeyModifier
/* Enumerates different platform-independent virtual keys on a physical keyboard whose input via keyboard may or may not produce characters. */
enum class VirtualKey {
    Shift = 36,
    Control = 35,
    Alternate = 39,
    Meta = 37,
    Command = 40,
    Cancel = 41,
    Help = 42,
    Backspace = 43,
    Tab = 3,
    Clear = 44,
    Enter = 45,
    Pause = 46,
    Escape = 47,
    PageUp = 48,
    PageDown = 49,
    End = 50,
    Home = 51,
    LeftArrow = 52,
    UpArrow = 53,
    RightArrow = 54,
    DownArrow = 55,
    Insert = 56,
    Delete = 57,
    Space = 58,
    Semicolon = 59,
    Equals = 60,
    Return = 61,
    NumberPad0 = 62,
    NumberPad1 = 63,
    NumberPad2 = 64,
    NumberPad3 = 65,
    NumberPad4 = 66,
    NumberPad5 = 67,
    NumberPad6 = 68,
    NumberPad7 = 69,
    NumberPad8 = 70,
    NumberPad9 = 71,
    NumberPadMultiply = 72,
    NumberPadAdd = 73,
    NumberPadSeparator = 74,
    NumberPadSubtract = 75,
    NumberPadDecimal = 76,
    NumberPadDivide = 77,
    Function1 = 78,
    Function2 = 79,
    Function3 = 80,
    Function4 = 81,
    Function5 = 82,
    Function6 = 83,
    Function7 = 84,
    Function8 = 85,
    Function9 = 86,
    Function10 = 87,
    Function11 = 88,
    Function12 = 89,
}; // enum class VirtualKey
/* Enumerates different ways of interacting with a keyboard device. 'InsertByKey' implies that a separate KeyDown and KeyUp event are produced for each combining character sequence, regardless of the actual keystrokes required to produce the character sequence. */
enum class KeyboardInteractionType {
    KeyPress = 90,
    KeyRelease = 91,
    InsertByKey = 92,
}; // enum class KeyboardInteractionType
/* A single step in a key sequence. A step can contain a key up/down of a virtual key, or a sequence of Unicode code points that are delivered to the page at grapheme cluster boundaries. Either a 'key' or 'text' property must be specified. */
class KeyboardInteraction : public JSON::ObjectBase {
public:
    enum {
        NoFieldsSet = 0,
        TypeSet = 1 << 0,
        AllFieldsSet = (TypeSet)
    };

    template<int STATE>
    class Builder {
    private:
        RefPtr<JSON::Object> m_result;

        template<int STEP> Builder<STATE | STEP>& castState()
        {
            return *reinterpret_cast<Builder<STATE | STEP>*>(this);
        }

        Builder(Ref</*KeyboardInteraction*/JSON::Object>&& object)
            : m_result(WTFMove(object))
        {
            COMPILE_ASSERT(STATE == NoFieldsSet, builder_created_in_non_init_state);
        }
        friend class KeyboardInteraction;
    public:

        Builder<STATE | TypeSet>& setType(Inspector::Protocol::Automation::KeyboardInteractionType value)
        {
            COMPILE_ASSERT(!(STATE & TypeSet), property_type_already_set);
            m_result->setString("type"_s, Inspector::Protocol::AutomationHelpers::getEnumConstantValue(value));
            return castState<TypeSet>();
        }

        Ref<KeyboardInteraction> release()
        {
            COMPILE_ASSERT(STATE == AllFieldsSet, result_is_not_ready);
            COMPILE_ASSERT(sizeof(KeyboardInteraction) == sizeof(JSON::Object), cannot_cast);

            Ref<JSON::Object> jsonResult = m_result.releaseNonNull();
            auto result = WTFMove(*reinterpret_cast<Ref<KeyboardInteraction>*>(&jsonResult));
            return result;
        }
    };

    /*
     * Synthetic constructor:
     * Ref<KeyboardInteraction> result = KeyboardInteraction::create()
     *     .setType(...)
     *     .release();
     */
    static Builder<NoFieldsSet> create()
    {
        return Builder<NoFieldsSet>(JSON::Object::create());
    }

    void setKey(Inspector::Protocol::Automation::VirtualKey value)
    {
        JSON::ObjectBase::setString("key"_s, Inspector::Protocol::AutomationHelpers::getEnumConstantValue(value));
    }

    void setText(const String& value)
    {
        JSON::ObjectBase::setString("text"_s, value);
    }
};

class Cookie : public JSON::ObjectBase {
public:
    enum {
        NoFieldsSet = 0,
        NameSet = 1 << 0,
        ValueSet = 1 << 1,
        DomainSet = 1 << 2,
        PathSet = 1 << 3,
        ExpiresSet = 1 << 4,
        SizeSet = 1 << 5,
        HttpOnlySet = 1 << 6,
        SecureSet = 1 << 7,
        SessionSet = 1 << 8,
        AllFieldsSet = (NameSet | ValueSet | DomainSet | PathSet | ExpiresSet | SizeSet | HttpOnlySet | SecureSet | SessionSet)
    };

    template<int STATE>
    class Builder {
    private:
        RefPtr<JSON::Object> m_result;

        template<int STEP> Builder<STATE | STEP>& castState()
        {
            return *reinterpret_cast<Builder<STATE | STEP>*>(this);
        }

        Builder(Ref</*Cookie*/JSON::Object>&& object)
            : m_result(WTFMove(object))
        {
            COMPILE_ASSERT(STATE == NoFieldsSet, builder_created_in_non_init_state);
        }
        friend class Cookie;
    public:

        Builder<STATE | NameSet>& setName(const String& value)
        {
            COMPILE_ASSERT(!(STATE & NameSet), property_name_already_set);
            m_result->setString("name"_s, value);
            return castState<NameSet>();
        }

        Builder<STATE | ValueSet>& setValue(const String& value)
        {
            COMPILE_ASSERT(!(STATE & ValueSet), property_value_already_set);
            m_result->setString("value"_s, value);
            return castState<ValueSet>();
        }

        Builder<STATE | DomainSet>& setDomain(const String& value)
        {
            COMPILE_ASSERT(!(STATE & DomainSet), property_domain_already_set);
            m_result->setString("domain"_s, value);
            return castState<DomainSet>();
        }

        Builder<STATE | PathSet>& setPath(const String& value)
        {
            COMPILE_ASSERT(!(STATE & PathSet), property_path_already_set);
            m_result->setString("path"_s, value);
            return castState<PathSet>();
        }

        Builder<STATE | ExpiresSet>& setExpires(double value)
        {
            COMPILE_ASSERT(!(STATE & ExpiresSet), property_expires_already_set);
            m_result->setDouble("expires"_s, value);
            return castState<ExpiresSet>();
        }

        Builder<STATE | SizeSet>& setSize(int value)
        {
            COMPILE_ASSERT(!(STATE & SizeSet), property_size_already_set);
            m_result->setInteger("size"_s, value);
            return castState<SizeSet>();
        }

        Builder<STATE | HttpOnlySet>& setHttpOnly(bool value)
        {
            COMPILE_ASSERT(!(STATE & HttpOnlySet), property_httpOnly_already_set);
            m_result->setBoolean("httpOnly"_s, value);
            return castState<HttpOnlySet>();
        }

        Builder<STATE | SecureSet>& setSecure(bool value)
        {
            COMPILE_ASSERT(!(STATE & SecureSet), property_secure_already_set);
            m_result->setBoolean("secure"_s, value);
            return castState<SecureSet>();
        }

        Builder<STATE | SessionSet>& setSession(bool value)
        {
            COMPILE_ASSERT(!(STATE & SessionSet), property_session_already_set);
            m_result->setBoolean("session"_s, value);
            return castState<SessionSet>();
        }

        Ref<Cookie> release()
        {
            COMPILE_ASSERT(STATE == AllFieldsSet, result_is_not_ready);
            COMPILE_ASSERT(sizeof(Cookie) == sizeof(JSON::Object), cannot_cast);

            Ref<JSON::Object> jsonResult = m_result.releaseNonNull();
            auto result = WTFMove(*reinterpret_cast<Ref<Cookie>*>(&jsonResult));
            return result;
        }
    };

    /*
     * Synthetic constructor:
     * Ref<Cookie> result = Cookie::create()
     *     .setName(...)
     *     .setValue(...)
     *     .setDomain(...)
     *     .setPath(...)
     *     .setExpires(...)
     *     .setSize(...)
     *     .setHttpOnly(...)
     *     .setSecure(...)
     *     .setSession(...)
     *     .release();
     */
    static Builder<NoFieldsSet> create()
    {
        return Builder<NoFieldsSet>(JSON::Object::create());
    }
};

/* Enumerates different permissions that must be obtained in order to use privileged APIs. */
enum class SessionPermission {
    GetUserMedia = 93,
}; // enum class SessionPermission
class SessionPermissionData : public JSON::ObjectBase {
public:
    enum {
        NoFieldsSet = 0,
        PermissionSet = 1 << 0,
        ValueSet = 1 << 1,
        AllFieldsSet = (PermissionSet | ValueSet)
    };

    template<int STATE>
    class Builder {
    private:
        RefPtr<JSON::Object> m_result;

        template<int STEP> Builder<STATE | STEP>& castState()
        {
            return *reinterpret_cast<Builder<STATE | STEP>*>(this);
        }

        Builder(Ref</*SessionPermissionData*/JSON::Object>&& object)
            : m_result(WTFMove(object))
        {
            COMPILE_ASSERT(STATE == NoFieldsSet, builder_created_in_non_init_state);
        }
        friend class SessionPermissionData;
    public:

        Builder<STATE | PermissionSet>& setPermission(Inspector::Protocol::Automation::SessionPermission value)
        {
            COMPILE_ASSERT(!(STATE & PermissionSet), property_permission_already_set);
            m_result->setString("permission"_s, Inspector::Protocol::AutomationHelpers::getEnumConstantValue(value));
            return castState<PermissionSet>();
        }

        Builder<STATE | ValueSet>& setValue(bool value)
        {
            COMPILE_ASSERT(!(STATE & ValueSet), property_value_already_set);
            m_result->setBoolean("value"_s, value);
            return castState<ValueSet>();
        }

        Ref<SessionPermissionData> release()
        {
            COMPILE_ASSERT(STATE == AllFieldsSet, result_is_not_ready);
            COMPILE_ASSERT(sizeof(SessionPermissionData) == sizeof(JSON::Object), cannot_cast);

            Ref<JSON::Object> jsonResult = m_result.releaseNonNull();
            auto result = WTFMove(*reinterpret_cast<Ref<SessionPermissionData>*>(&jsonResult));
            return result;
        }
    };

    /*
     * Synthetic constructor:
     * Ref<SessionPermissionData> result = SessionPermissionData::create()
     *     .setPermission(...)
     *     .setValue(...)
     *     .release();
     */
    static Builder<NoFieldsSet> create()
    {
        return Builder<NoFieldsSet>(JSON::Object::create());
    }
};

/* The type of an input source. This determines which state properties are applicable to the input source. The 'Null' input source type has no properties, only a duration. This can extend the effective wait for a tick. */
enum class InputSourceType {
    Null = 94,
    Mouse = 95,
    Keyboard = 96,
    Touch = 97,
}; // enum class InputSourceType
class InputSource : public JSON::ObjectBase {
public:
    enum {
        NoFieldsSet = 0,
        SourceIdSet = 1 << 0,
        SourceTypeSet = 1 << 1,
        AllFieldsSet = (SourceIdSet | SourceTypeSet)
    };

    template<int STATE>
    class Builder {
    private:
        RefPtr<JSON::Object> m_result;

        template<int STEP> Builder<STATE | STEP>& castState()
        {
            return *reinterpret_cast<Builder<STATE | STEP>*>(this);
        }

        Builder(Ref</*InputSource*/JSON::Object>&& object)
            : m_result(WTFMove(object))
        {
            COMPILE_ASSERT(STATE == NoFieldsSet, builder_created_in_non_init_state);
        }
        friend class InputSource;
    public:

        Builder<STATE | SourceIdSet>& setSourceId(const String& value)
        {
            COMPILE_ASSERT(!(STATE & SourceIdSet), property_sourceId_already_set);
            m_result->setString("sourceId"_s, value);
            return castState<SourceIdSet>();
        }

        Builder<STATE | SourceTypeSet>& setSourceType(Inspector::Protocol::Automation::InputSourceType value)
        {
            COMPILE_ASSERT(!(STATE & SourceTypeSet), property_sourceType_already_set);
            m_result->setString("sourceType"_s, Inspector::Protocol::AutomationHelpers::getEnumConstantValue(value));
            return castState<SourceTypeSet>();
        }

        Ref<InputSource> release()
        {
            COMPILE_ASSERT(STATE == AllFieldsSet, result_is_not_ready);
            COMPILE_ASSERT(sizeof(InputSource) == sizeof(JSON::Object), cannot_cast);

            Ref<JSON::Object> jsonResult = m_result.releaseNonNull();
            auto result = WTFMove(*reinterpret_cast<Ref<InputSource>*>(&jsonResult));
            return result;
        }
    };

    /*
     * Synthetic constructor:
     * Ref<InputSource> result = InputSource::create()
     *     .setSourceId(...)
     *     .setSourceType(...)
     *     .release();
     */
    static Builder<NoFieldsSet> create()
    {
        return Builder<NoFieldsSet>(JSON::Object::create());
    }
};

/* A set of input source states to which all applicable input sources must transition before further steps may be processed. If an input source does not have a corresponding state in a step, its state assumed to be 'released' for that step. */
class InteractionStep : public JSON::ObjectBase {
public:
    enum {
        NoFieldsSet = 0,
        AllFieldsSet = 0
    };

    template<int STATE>
    class Builder {
    private:
        RefPtr<JSON::Object> m_result;

        template<int STEP> Builder<STATE | STEP>& castState()
        {
            return *reinterpret_cast<Builder<STATE | STEP>*>(this);
        }

        Builder(Ref</*InteractionStep*/JSON::Object>&& object)
            : m_result(WTFMove(object))
        {
            COMPILE_ASSERT(STATE == NoFieldsSet, builder_created_in_non_init_state);
        }
        friend class InteractionStep;
    public:

        Ref<InteractionStep> release()
        {
            COMPILE_ASSERT(STATE == AllFieldsSet, result_is_not_ready);
            COMPILE_ASSERT(sizeof(InteractionStep) == sizeof(JSON::Object), cannot_cast);

            Ref<JSON::Object> jsonResult = m_result.releaseNonNull();
            auto result = WTFMove(*reinterpret_cast<Ref<InteractionStep>*>(&jsonResult));
            return result;
        }
    };

    /*
     * Synthetic constructor:
     * Ref<InteractionStep> result = InteractionStep::create()
     *     .release();
     */
    static Builder<NoFieldsSet> create()
    {
        return Builder<NoFieldsSet>(JSON::Object::create());
    }

    void setStates(RefPtr<JSON::ArrayOf<Inspector::Protocol::Automation::InputSourceState>> value)
    {
        JSON::ObjectBase::setArray("states"_s, WTFMove(value));
    }
};

/* Enumerates different origin types that can be used in mouse move interactions. */
enum class MouseMoveOrigin {
    Viewport = 1,
    Pointer = 98,
    Element = 99,
}; // enum class MouseMoveOrigin
/* A new state for a specific input source. All state-related fields are optional and must be applicable to the InputSource referenced by 'sourceId'. If no state-related fields are specified, the state is assumed to remain the same as in the previous step (i.e., 'sustained'). */
class InputSourceState : public JSON::ObjectBase {
public:
    enum {
        NoFieldsSet = 0,
        SourceIdSet = 1 << 0,
        AllFieldsSet = (SourceIdSet)
    };

    template<int STATE>
    class Builder {
    private:
        RefPtr<JSON::Object> m_result;

        template<int STEP> Builder<STATE | STEP>& castState()
        {
            return *reinterpret_cast<Builder<STATE | STEP>*>(this);
        }

        Builder(Ref</*InputSourceState*/JSON::Object>&& object)
            : m_result(WTFMove(object))
        {
            COMPILE_ASSERT(STATE == NoFieldsSet, builder_created_in_non_init_state);
        }
        friend class InputSourceState;
    public:

        Builder<STATE | SourceIdSet>& setSourceId(const String& value)
        {
            COMPILE_ASSERT(!(STATE & SourceIdSet), property_sourceId_already_set);
            m_result->setString("sourceId"_s, value);
            return castState<SourceIdSet>();
        }

        Ref<InputSourceState> release()
        {
            COMPILE_ASSERT(STATE == AllFieldsSet, result_is_not_ready);
            COMPILE_ASSERT(sizeof(InputSourceState) == sizeof(JSON::Object), cannot_cast);

            Ref<JSON::Object> jsonResult = m_result.releaseNonNull();
            auto result = WTFMove(*reinterpret_cast<Ref<InputSourceState>*>(&jsonResult));
            return result;
        }
    };

    /*
     * Synthetic constructor:
     * Ref<InputSourceState> result = InputSourceState::create()
     *     .setSourceId(...)
     *     .release();
     */
    static Builder<NoFieldsSet> create()
    {
        return Builder<NoFieldsSet>(JSON::Object::create());
    }

    void setPressedCharKey(const String& value)
    {
        JSON::ObjectBase::setString("pressedCharKey"_s, value);
    }

    void setPressedVirtualKeys(RefPtr<JSON::ArrayOf<Inspector::Protocol::Automation::VirtualKey>> value)
    {
        JSON::ObjectBase::setArray("pressedVirtualKeys"_s, WTFMove(value));
    }

    void setPressedButton(Inspector::Protocol::Automation::MouseButton value)
    {
        JSON::ObjectBase::setString("pressedButton"_s, Inspector::Protocol::AutomationHelpers::getEnumConstantValue(value));
    }

    void setOrigin(Inspector::Protocol::Automation::MouseMoveOrigin value)
    {
        JSON::ObjectBase::setString("origin"_s, Inspector::Protocol::AutomationHelpers::getEnumConstantValue(value));
    }

    void setNodeHandle(const String& value)
    {
        JSON::ObjectBase::setString("nodeHandle"_s, value);
    }

    void setLocation(RefPtr<Inspector::Protocol::Automation::Point> value)
    {
        JSON::ObjectBase::setObject("location"_s, WTFMove(value));
    }

    void setDuration(int value)
    {
        JSON::ObjectBase::setInteger("duration"_s, value);
    }
};

} // Automation



namespace AutomationHelpers {

template<typename ProtocolEnumType>
Optional<ProtocolEnumType> parseEnumValueFromString(const String&);

// Enums in the 'Automation' Domain
template<>
Optional<Inspector::Protocol::Automation::CoordinateSystem> parseEnumValueFromString<Inspector::Protocol::Automation::CoordinateSystem>(const String&);
template<>
Optional<Inspector::Protocol::Automation::BrowsingContextPresentation> parseEnumValueFromString<Inspector::Protocol::Automation::BrowsingContextPresentation>(const String&);
template<>
Optional<Inspector::Protocol::Automation::ErrorMessage> parseEnumValueFromString<Inspector::Protocol::Automation::ErrorMessage>(const String&);
template<>
Optional<Inspector::Protocol::Automation::PageLoadStrategy> parseEnumValueFromString<Inspector::Protocol::Automation::PageLoadStrategy>(const String&);
template<>
Optional<Inspector::Protocol::Automation::MouseInteraction> parseEnumValueFromString<Inspector::Protocol::Automation::MouseInteraction>(const String&);
template<>
Optional<Inspector::Protocol::Automation::MouseButton> parseEnumValueFromString<Inspector::Protocol::Automation::MouseButton>(const String&);
template<>
Optional<Inspector::Protocol::Automation::KeyModifier> parseEnumValueFromString<Inspector::Protocol::Automation::KeyModifier>(const String&);
template<>
Optional<Inspector::Protocol::Automation::VirtualKey> parseEnumValueFromString<Inspector::Protocol::Automation::VirtualKey>(const String&);
template<>
Optional<Inspector::Protocol::Automation::KeyboardInteractionType> parseEnumValueFromString<Inspector::Protocol::Automation::KeyboardInteractionType>(const String&);
template<>
Optional<Inspector::Protocol::Automation::SessionPermission> parseEnumValueFromString<Inspector::Protocol::Automation::SessionPermission>(const String&);
template<>
Optional<Inspector::Protocol::Automation::InputSourceType> parseEnumValueFromString<Inspector::Protocol::Automation::InputSourceType>(const String&);
template<>
Optional<Inspector::Protocol::Automation::MouseMoveOrigin> parseEnumValueFromString<Inspector::Protocol::Automation::MouseMoveOrigin>(const String&);

} // namespace AutomationHelpers

} // namespace Protocol

} // namespace Inspector

namespace WTF {

template<typename T> struct DefaultHash;

// Hash declarations in the 'Automation' Domain
template<>
struct DefaultHash<Inspector::Protocol::Automation::CoordinateSystem> {
    typedef IntHash<Inspector::Protocol::Automation::CoordinateSystem> Hash;
};
template<>
struct DefaultHash<Inspector::Protocol::Automation::BrowsingContextPresentation> {
    typedef IntHash<Inspector::Protocol::Automation::BrowsingContextPresentation> Hash;
};
template<>
struct DefaultHash<Inspector::Protocol::Automation::ErrorMessage> {
    typedef IntHash<Inspector::Protocol::Automation::ErrorMessage> Hash;
};
template<>
struct DefaultHash<Inspector::Protocol::Automation::PageLoadStrategy> {
    typedef IntHash<Inspector::Protocol::Automation::PageLoadStrategy> Hash;
};
template<>
struct DefaultHash<Inspector::Protocol::Automation::MouseInteraction> {
    typedef IntHash<Inspector::Protocol::Automation::MouseInteraction> Hash;
};
template<>
struct DefaultHash<Inspector::Protocol::Automation::MouseButton> {
    typedef IntHash<Inspector::Protocol::Automation::MouseButton> Hash;
};
template<>
struct DefaultHash<Inspector::Protocol::Automation::KeyModifier> {
    typedef IntHash<Inspector::Protocol::Automation::KeyModifier> Hash;
};
template<>
struct DefaultHash<Inspector::Protocol::Automation::VirtualKey> {
    typedef IntHash<Inspector::Protocol::Automation::VirtualKey> Hash;
};
template<>
struct DefaultHash<Inspector::Protocol::Automation::KeyboardInteractionType> {
    typedef IntHash<Inspector::Protocol::Automation::KeyboardInteractionType> Hash;
};
template<>
struct DefaultHash<Inspector::Protocol::Automation::SessionPermission> {
    typedef IntHash<Inspector::Protocol::Automation::SessionPermission> Hash;
};
template<>
struct DefaultHash<Inspector::Protocol::Automation::InputSourceType> {
    typedef IntHash<Inspector::Protocol::Automation::InputSourceType> Hash;
};
template<>
struct DefaultHash<Inspector::Protocol::Automation::MouseMoveOrigin> {
    typedef IntHash<Inspector::Protocol::Automation::MouseMoveOrigin> Hash;
};

} // namespace WTF
