/*
 * Copyright (C) 2010-2019 Apple Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1.  Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 * 2.  Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY APPLE INC. AND ITS CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL APPLE INC. OR ITS CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "config.h"

#include "AuxiliaryProcess.h"

#include "AuxiliaryProcessMessages.h"
#include "Decoder.h"
#include "HandleMessage.h"

namespace WebKit {

void AuxiliaryProcess::didReceiveMessage(IPC::Connection& connection, IPC::Decoder& decoder)
{
    if (decoder.messageName() == Messages::AuxiliaryProcess::ShutDown::name()) {
        IPC::handleMessage<Messages::AuxiliaryProcess::ShutDown>(decoder, this, &AuxiliaryProcess::shutDown);
        return;
    }
    if (decoder.messageName() == Messages::AuxiliaryProcess::SetProcessSuppressionEnabled::name()) {
        IPC::handleMessage<Messages::AuxiliaryProcess::SetProcessSuppressionEnabled>(decoder, this, &AuxiliaryProcess::setProcessSuppressionEnabled);
        return;
    }
#if OS(LINUX)
    if (decoder.messageName() == Messages::AuxiliaryProcess::DidReceiveMemoryPressureEvent::name()) {
        IPC::handleMessage<Messages::AuxiliaryProcess::DidReceiveMemoryPressureEvent>(decoder, this, &AuxiliaryProcess::didReceiveMemoryPressureEvent);
        return;
    }
#endif
    UNUSED_PARAM(connection);
    UNUSED_PARAM(decoder);
    ASSERT_NOT_REACHED();
}

} // namespace WebKit

