/*
 * Copyright (C) 2010-2019 Apple Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1.  Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 * 2.  Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY APPLE INC. AND ITS CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL APPLE INC. OR ITS CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#pragma once

#if ENABLE(CONTENT_EXTENSIONS)

#include "ArgumentCoders.h"
#include "Connection.h"
#include "NetworkContentRuleListManagerMessagesReplies.h"
#include "UserContentControllerIdentifier.h"
#include "WebCompiledContentRuleListData.h"
#include <utility>
#include <wtf/Forward.h>
#include <wtf/ThreadSafeRefCounted.h>
#include <wtf/Vector.h>
#include <wtf/text/WTFString.h>


namespace Messages {
namespace NetworkContentRuleListManager {

static inline IPC::StringReference messageReceiverName()
{
    return IPC::StringReference("NetworkContentRuleListManager");
}

class Remove {
public:
    typedef std::tuple<const WebKit::UserContentControllerIdentifier&> Arguments;

    static IPC::StringReference receiverName() { return messageReceiverName(); }
    static IPC::StringReference name() { return IPC::StringReference("Remove"); }
    static const bool isSync = false;

    explicit Remove(const WebKit::UserContentControllerIdentifier& identifier)
        : m_arguments(identifier)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

class AddContentRuleLists {
public:
    typedef std::tuple<const WebKit::UserContentControllerIdentifier&, const Vector<std::pair<String, WebKit::WebCompiledContentRuleListData>>&> Arguments;

    static IPC::StringReference receiverName() { return messageReceiverName(); }
    static IPC::StringReference name() { return IPC::StringReference("AddContentRuleLists"); }
    static const bool isSync = false;

    AddContentRuleLists(const WebKit::UserContentControllerIdentifier& identifier, const Vector<std::pair<String, WebKit::WebCompiledContentRuleListData>>& contentFilters)
        : m_arguments(identifier, contentFilters)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

class RemoveContentRuleList {
public:
    typedef std::tuple<const WebKit::UserContentControllerIdentifier&, const String&> Arguments;

    static IPC::StringReference receiverName() { return messageReceiverName(); }
    static IPC::StringReference name() { return IPC::StringReference("RemoveContentRuleList"); }
    static const bool isSync = false;

    RemoveContentRuleList(const WebKit::UserContentControllerIdentifier& identifier, const String& name)
        : m_arguments(identifier, name)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

class RemoveAllContentRuleLists {
public:
    typedef std::tuple<const WebKit::UserContentControllerIdentifier&> Arguments;

    static IPC::StringReference receiverName() { return messageReceiverName(); }
    static IPC::StringReference name() { return IPC::StringReference("RemoveAllContentRuleLists"); }
    static const bool isSync = false;

    explicit RemoveAllContentRuleLists(const WebKit::UserContentControllerIdentifier& identifier)
        : m_arguments(identifier)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

} // namespace NetworkContentRuleListManager
} // namespace Messages

#endif // ENABLE(CONTENT_EXTENSIONS)
