/*
 * Copyright (C) 2010-2019 Apple Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1.  Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 * 2.  Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY APPLE INC. AND ITS CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL APPLE INC. OR ITS CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "config.h"

#if ENABLE(SERVICE_WORKER)

#include "ServiceWorkerFetchTask.h"

#include "DataReference.h"
#include "Decoder.h"
#include "FormDataReference.h"
#include "HandleMessage.h"
#include "ServiceWorkerFetchTaskMessages.h"
#include "SharedBufferDataReference.h"
#include "WebCoreArgumentCoders.h"
#include <WebCore/ResourceError.h>
#include <WebCore/ResourceResponse.h>

namespace WebKit {

void ServiceWorkerFetchTask::didReceiveMessage(IPC::Connection& connection, IPC::Decoder& decoder)
{
    if (decoder.messageName() == Messages::ServiceWorkerFetchTask::DidNotHandle::name()) {
        IPC::handleMessage<Messages::ServiceWorkerFetchTask::DidNotHandle>(decoder, this, &ServiceWorkerFetchTask::didNotHandle);
        return;
    }
    if (decoder.messageName() == Messages::ServiceWorkerFetchTask::DidFail::name()) {
        IPC::handleMessage<Messages::ServiceWorkerFetchTask::DidFail>(decoder, this, &ServiceWorkerFetchTask::didFail);
        return;
    }
    if (decoder.messageName() == Messages::ServiceWorkerFetchTask::DidReceiveRedirectResponse::name()) {
        IPC::handleMessage<Messages::ServiceWorkerFetchTask::DidReceiveRedirectResponse>(decoder, this, &ServiceWorkerFetchTask::didReceiveRedirectResponse);
        return;
    }
    if (decoder.messageName() == Messages::ServiceWorkerFetchTask::DidReceiveResponse::name()) {
        IPC::handleMessage<Messages::ServiceWorkerFetchTask::DidReceiveResponse>(decoder, this, &ServiceWorkerFetchTask::didReceiveResponse);
        return;
    }
    if (decoder.messageName() == Messages::ServiceWorkerFetchTask::DidReceiveData::name()) {
        IPC::handleMessage<Messages::ServiceWorkerFetchTask::DidReceiveData>(decoder, this, &ServiceWorkerFetchTask::didReceiveData);
        return;
    }
    if (decoder.messageName() == Messages::ServiceWorkerFetchTask::DidReceiveFormData::name()) {
        IPC::handleMessage<Messages::ServiceWorkerFetchTask::DidReceiveFormData>(decoder, this, &ServiceWorkerFetchTask::didReceiveFormData);
        return;
    }
    if (decoder.messageName() == Messages::ServiceWorkerFetchTask::DidFinish::name()) {
        IPC::handleMessage<Messages::ServiceWorkerFetchTask::DidFinish>(decoder, this, &ServiceWorkerFetchTask::didFinish);
        return;
    }
    UNUSED_PARAM(connection);
    UNUSED_PARAM(decoder);
    ASSERT_NOT_REACHED();
}

} // namespace WebKit


#endif // ENABLE(SERVICE_WORKER)
