/*
 * Copyright (C) 2010-2019 Apple Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1.  Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 * 2.  Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY APPLE INC. AND ITS CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL APPLE INC. OR ITS CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "config.h"

#include "ViewGestureGeometryCollector.h"

#include "Decoder.h"
#include "HandleMessage.h"
#include "ViewGestureGeometryCollectorMessages.h"
#if PLATFORM(COCOA)
#include "WebCoreArgumentCoders.h"
#endif
#if PLATFORM(COCOA)
#include <WebCore/FloatPoint.h>
#endif

namespace WebKit {

void ViewGestureGeometryCollector::didReceiveMessage(IPC::Connection& connection, IPC::Decoder& decoder)
{
#if PLATFORM(COCOA)
    if (decoder.messageName() == Messages::ViewGestureGeometryCollector::CollectGeometryForSmartMagnificationGesture::name()) {
        IPC::handleMessage<Messages::ViewGestureGeometryCollector::CollectGeometryForSmartMagnificationGesture>(decoder, this, &ViewGestureGeometryCollector::collectGeometryForSmartMagnificationGesture);
        return;
    }
#endif
#if PLATFORM(MAC)
    if (decoder.messageName() == Messages::ViewGestureGeometryCollector::CollectGeometryForMagnificationGesture::name()) {
        IPC::handleMessage<Messages::ViewGestureGeometryCollector::CollectGeometryForMagnificationGesture>(decoder, this, &ViewGestureGeometryCollector::collectGeometryForMagnificationGesture);
        return;
    }
#endif
#if !PLATFORM(IOS_FAMILY)
    if (decoder.messageName() == Messages::ViewGestureGeometryCollector::SetRenderTreeSizeNotificationThreshold::name()) {
        IPC::handleMessage<Messages::ViewGestureGeometryCollector::SetRenderTreeSizeNotificationThreshold>(decoder, this, &ViewGestureGeometryCollector::setRenderTreeSizeNotificationThreshold);
        return;
    }
#endif
    UNUSED_PARAM(connection);
    UNUSED_PARAM(decoder);
    ASSERT_NOT_REACHED();
}

} // namespace WebKit

