/*
 * Copyright (C) 2010-2019 Apple Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1.  Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 * 2.  Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY APPLE INC. AND ITS CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL APPLE INC. OR ITS CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#pragma once

#include "SharedMemory.h"
#include <WebCore/ElementContext.h>
#include <WebCore/FloatRect.h>
#include <WebCore/IntRect.h>
#include <WebCore/TextIndicator.h>
#include <WebCore/TextManipulationController.h>
#include <wtf/Forward.h>
#include <wtf/Optional.h>
#include <wtf/Vector.h>
#include <wtf/text/WTFString.h>

namespace WebCore {
class IntPoint;
}

namespace WebKit {
struct AttributedString;
struct DataDetectionResult;
struct DocumentEditingContext;
struct InteractionInformationAtPosition;
struct UserMessage;
struct WebAutocorrectionData;
}

namespace Messages {
namespace WebPage {

using TestProcessIncomingSyncMessagesWhenWaitingForSyncReplyDelayedReply = CompletionHandler<void(bool handled)>;

#if PLATFORM(IOS_FAMILY)
using RequestAutocorrectionDataAsyncReply = CompletionHandler<void(const WebKit::WebAutocorrectionData& data)>;
#endif

#if PLATFORM(IOS_FAMILY)
using SyncApplyAutocorrectionDelayedReply = CompletionHandler<void(bool autocorrectionApplied)>;
#endif

#if PLATFORM(IOS_FAMILY)
using RequestEvasionRectsAboveSelectionAsyncReply = CompletionHandler<void(const Vector<WebCore::FloatRect>& rects)>;
#endif

#if PLATFORM(IOS_FAMILY)
using GetPositionInformationDelayedReply = CompletionHandler<void(const WebKit::InteractionInformationAtPosition& information)>;
#endif

#if PLATFORM(IOS_FAMILY)
using UpdateSelectionWithDeltaAsyncReply = CompletionHandler<void()>;
#endif

#if PLATFORM(IOS_FAMILY)
using RequestDocumentEditingContextAsyncReply = CompletionHandler<void(const WebKit::DocumentEditingContext& response)>;
#endif

#if ENABLE(IOS_TOUCH_EVENTS)
using TouchEventSyncDelayedReply = CompletionHandler<void(bool handled)>;
#endif

#if PLATFORM(COCOA)
using GetContentsAsAttributedStringAsyncReply = CompletionHandler<void(const WebKit::AttributedString& result)>;
#endif

#if ENABLE(DATA_DETECTION)
using DetectDataInAllFramesAsyncReply = CompletionHandler<void(const WebKit::DataDetectionResult& result)>;
#endif

#if ENABLE(DATA_DETECTION)
using RemoveDataDetectedLinksAsyncReply = CompletionHandler<void(const WebKit::DataDetectionResult& result)>;
#endif

using TryCloseAsyncReply = CompletionHandler<void(bool shouldClose)>;

#if PLATFORM(IOS_FAMILY) && ENABLE(DRAG_SUPPORT)
using InsertDroppedImagePlaceholdersAsyncReply = CompletionHandler<void(const Vector<WebCore::IntRect>& dropPreviewRects, const Optional<WebCore::TextIndicatorData>& textIndicator)>;
#endif

#if ENABLE(MEDIA_STREAM)
using UserMediaAccessWasGrantedAsyncReply = CompletionHandler<void()>;
#endif

#if (PLATFORM(COCOA) && PLATFORM(IOS_FAMILY))
using ComputePagesForPrintingAndDrawToPDFDelayedReply = CompletionHandler<void(uint32_t pageCount)>;
#endif

#if PLATFORM(GTK)
using GetCenterForZoomGestureDelayedReply = CompletionHandler<void(const WebCore::IntPoint& center)>;
#endif

#if PLATFORM(COCOA)
using GetStringSelectionForPasteboardDelayedReply = CompletionHandler<void(const String& stringValue)>;
#endif

#if PLATFORM(COCOA)
using GetDataSelectionForPasteboardDelayedReply = CompletionHandler<void(const WebKit::SharedMemory::Handle& handle, uint64_t size)>;
#endif

#if PLATFORM(COCOA)
using ReadSelectionFromPasteboardDelayedReply = CompletionHandler<void(bool result)>;
#endif

#if PLATFORM(COCOA)
using ShouldDelayWindowOrderingEventDelayedReply = CompletionHandler<void(bool result)>;
#endif

#if PLATFORM(COCOA)
using AcceptsFirstMouseDelayedReply = CompletionHandler<void(bool result)>;
#endif

#if PLATFORM(COCOA)
using HasMarkedTextAsyncReply = CompletionHandler<void(bool hasMarkedText)>;
#endif

using TextInputContextsInRectAsyncReply = CompletionHandler<void(const Vector<struct WebCore::ElementContext>& contexts)>;

using FocusTextInputContextAsyncReply = CompletionHandler<void(bool success)>;

#if PLATFORM(GTK) || PLATFORM(WPE)
using SendMessageToWebExtensionWithReplyAsyncReply = CompletionHandler<void(const WebKit::UserMessage& replyMessage)>;
#endif

using StartTextManipulationsAsyncReply = CompletionHandler<void()>;

using CompleteTextManipulationAsyncReply = CompletionHandler<void(WebCore::TextManipulationController::ManipulationResult result)>;

} // namespace WebPage
} // namespace Messages
