/*
 * Copyright (C) 2010-2019 Apple Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1.  Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 * 2.  Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY APPLE INC. AND ITS CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL APPLE INC. OR ITS CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#pragma once

#include "SandboxExtension.h"
#include "WebPageCreationParameters.h"
#include "WebSpeechSynthesisVoice.h"
#include "WebsitePoliciesData.h"
#include <WebCore/BackForwardItemIdentifier.h>
#include <WebCore/CaptureDevice.h>
#include <WebCore/PageIdentifier.h>
#include <WebCore/SearchPopupMenu.h>
#include <WebCore/SerializedAttachmentData.h>
#include <WebCore/TextCheckerClient.h>
#include <wtf/Forward.h>
#include <wtf/Optional.h>
#include <wtf/Vector.h>
#include <wtf/text/WTFString.h>

namespace WebCore {
class FloatRect;
class IntPoint;
class IntRect;
class PolicyCheckIdentifier;
class ResourceError;
class ResourceResponse;
enum class HasInsecureContent : bool;
enum class DOMPasteAccessResponse : uint8_t;
enum class DeviceOrientationOrMotionPermissionState : uint8_t;
enum class PolicyAction : uint8_t;
}

namespace WebKit {
class DownloadID;
class UserData;
struct UserMessage;
struct WebBackForwardListCounts;
}

namespace Messages {
namespace WebPageProxy {

using CreateNewPageDelayedReply = CompletionHandler<void(const Optional<WebCore::PageIdentifier>& newPageID, const Optional<WebKit::WebPageCreationParameters>& newPageParameters)>;

using RunJavaScriptAlertDelayedReply = CompletionHandler<void()>;

using RunJavaScriptConfirmDelayedReply = CompletionHandler<void(bool result)>;

using RunJavaScriptPromptDelayedReply = CompletionHandler<void(const String& result)>;

#if ENABLE(WEBGL)
using WebGLPolicyForURLDelayedReply = CompletionHandler<void(uint32_t loadPolicy)>;
#endif

#if ENABLE(WEBGL)
using ResolveWebGLPolicyForURLDelayedReply = CompletionHandler<void(uint32_t loadPolicy)>;
#endif

using GetToolbarsAreVisibleDelayedReply = CompletionHandler<void(bool toolbarsAreVisible)>;

using GetMenuBarIsVisibleDelayedReply = CompletionHandler<void(bool menuBarIsVisible)>;

using GetStatusBarIsVisibleDelayedReply = CompletionHandler<void(bool statusBarIsVisible)>;

using GetWindowFrameDelayedReply = CompletionHandler<void(const WebCore::FloatRect& windowFrame)>;

using ScreenToRootViewDelayedReply = CompletionHandler<void(const WebCore::IntPoint& windowPoint)>;

using RootViewToScreenDelayedReply = CompletionHandler<void(const WebCore::IntRect& screenFrame)>;

using AccessibilityScreenToRootViewDelayedReply = CompletionHandler<void(const WebCore::IntPoint& windowPoint)>;

using RootViewToAccessibilityScreenDelayedReply = CompletionHandler<void(const WebCore::IntRect& screenFrame)>;

using RunBeforeUnloadConfirmPanelDelayedReply = CompletionHandler<void(bool shouldClose)>;

using PrintFrameDelayedReply = CompletionHandler<void()>;

using DecidePolicyForNavigationActionSyncDelayedReply = CompletionHandler<void(const WebCore::PolicyCheckIdentifier& policyCheckIdentifier, WebCore::PolicyAction policyAction, uint64_t newNavigationID, const WebKit::DownloadID& downloadID, const Optional<WebKit::WebsitePoliciesData>& websitePolicies)>;

using HasInsecureContentDelayedReply = CompletionHandler<void(WebCore::HasInsecureContent hasInsecureContent)>;

#if PLATFORM(IOS_FAMILY)
using InterpretKeyEventDelayedReply = CompletionHandler<void(bool handled)>;
#endif

using BackForwardGoToItemDelayedReply = CompletionHandler<void(const WebKit::SandboxExtension::Handle& sandboxExtensionHandle, const WebKit::WebBackForwardListCounts& counts)>;

using BackForwardItemAtIndexDelayedReply = CompletionHandler<void(const Optional<WebCore::BackForwardItemIdentifier>& itemID)>;

using BackForwardListCountsDelayedReply = CompletionHandler<void(const WebKit::WebBackForwardListCounts& counts)>;

using CanUndoRedoDelayedReply = CompletionHandler<void(bool result)>;

using ExecuteUndoRedoDelayedReply = CompletionHandler<void()>;

using RequestDOMPasteAccessDelayedReply = CompletionHandler<void(WebCore::DOMPasteAccessResponse response)>;

using ExceededDatabaseQuotaDelayedReply = CompletionHandler<void(uint64_t newQuota)>;

using ReachedApplicationCacheOriginQuotaDelayedReply = CompletionHandler<void(uint64_t newQuota)>;

#if ENABLE(MEDIA_STREAM)
using EnumerateMediaDevicesForFrameAsyncReply = CompletionHandler<void(const Vector<WebCore::CaptureDevice>& devices, const String& mediaDeviceIdentifierHashSalt)>;
#endif

#if USE(UNIFIED_TEXT_CHECKING)
using CheckTextOfParagraphDelayedReply = CompletionHandler<void(const Vector<WebCore::TextCheckingResult>& results)>;
#endif

using CheckSpellingOfStringDelayedReply = CompletionHandler<void(int32_t misspellingLocation, int32_t misspellingLength)>;

using CheckGrammarOfStringDelayedReply = CompletionHandler<void(const Vector<WebCore::GrammarDetail>& results, int32_t badGrammarLocation, int32_t badGrammarLength)>;

using SpellingUIIsShowingDelayedReply = CompletionHandler<void(bool isShowing)>;

using GetGuessesForWordDelayedReply = CompletionHandler<void(const Vector<String>& guesses)>;

#if PLATFORM(COCOA)
using ExecuteSavedCommandBySelectorDelayedReply = CompletionHandler<void(bool handled)>;
#endif

#if PLATFORM(COCOA)
using GetIsSpeakingDelayedReply = CompletionHandler<void(bool isSpeaking)>;
#endif

#if USE(APPKIT)
using SubstitutionsPanelIsShowingDelayedReply = CompletionHandler<void(bool isShowing)>;
#endif

#if PLATFORM(MAC)
using DismissCorrectionPanelSoonDelayedReply = CompletionHandler<void(const String& result)>;
#endif

#if USE(DICTATION_ALTERNATIVES)
using DictationAlternativesDelayedReply = CompletionHandler<void(const Vector<String>& alternatives)>;
#endif

#if PLATFORM(X11)
using CreatePluginContainerDelayedReply = CompletionHandler<void(uint64_t windowID)>;
#endif

using LoadRecentSearchesDelayedReply = CompletionHandler<void(const Vector<WebCore::RecentSearch>& result)>;

#if ENABLE(PDFKIT_PLUGIN)
using ShowPDFContextMenuDelayedReply = CompletionHandler<void(const Optional<int32_t>& selectedItem)>;
#endif

#if ENABLE(NETSCAPE_PLUGIN_API)
using FindPluginDelayedReply = CompletionHandler<void(uint64_t pluginProcessToken, const String& newMIMEType, uint32_t pluginLoadPolicy, const String& unavailabilityDescription, bool isUnsupported)>;
#endif

#if ENABLE(WEB_CRYPTO)
using WrapCryptoKeyDelayedReply = CompletionHandler<void(bool succeeded, const Vector<uint8_t>& wrappedKey)>;
#endif

#if ENABLE(WEB_CRYPTO)
using UnwrapCryptoKeyDelayedReply = CompletionHandler<void(bool succeeded, const Vector<uint8_t>& key)>;
#endif

#if USE(QUICK_LOOK)
using RequestPasswordForQuickLookDocumentInMainFrameAsyncReply = CompletionHandler<void(const String& password)>;
#endif

using HandleSynchronousMessageDelayedReply = CompletionHandler<void(const WebKit::UserData& returnData)>;

using LoadSynchronousURLSchemeTaskDelayedReply = CompletionHandler<void(const WebCore::ResourceResponse& response, const WebCore::ResourceError& error, const Vector<char>& data)>;

#if ENABLE(DEVICE_ORIENTATION)
using ShouldAllowDeviceOrientationAndMotionAccessAsyncReply = CompletionHandler<void(WebCore::DeviceOrientationOrMotionPermissionState permissionState)>;
#endif

#if ENABLE(ATTACHMENT_ELEMENT)
using SerializedAttachmentDataForIdentifiersDelayedReply = CompletionHandler<void(const Vector<WebCore::SerializedAttachmentData>& seralizedData)>;
#endif

using SignedPublicKeyAndChallengeStringDelayedReply = CompletionHandler<void(const String& result)>;

#if ENABLE(SPEECH_SYNTHESIS)
using SpeechSynthesisVoiceListDelayedReply = CompletionHandler<void(const Vector<WebKit::WebSpeechSynthesisVoice>& voiceList)>;
#endif

#if ENABLE(SPEECH_SYNTHESIS)
using SpeechSynthesisSpeakAsyncReply = CompletionHandler<void()>;
#endif

#if ENABLE(SPEECH_SYNTHESIS)
using SpeechSynthesisSetFinishedCallbackAsyncReply = CompletionHandler<void()>;
#endif

#if ENABLE(SPEECH_SYNTHESIS)
using SpeechSynthesisPauseAsyncReply = CompletionHandler<void()>;
#endif

#if ENABLE(SPEECH_SYNTHESIS)
using SpeechSynthesisResumeAsyncReply = CompletionHandler<void()>;
#endif

#if PLATFORM(GTK)
using ShowEmojiPickerAsyncReply = CompletionHandler<void(const String& result)>;
#endif

#if PLATFORM(GTK) || PLATFORM(WPE)
using SendMessageToWebViewWithReplyAsyncReply = CompletionHandler<void(const WebKit::UserMessage& replyMessage)>;
#endif

} // namespace WebPageProxy
} // namespace Messages
