/*
 * Copyright (c) 2012 Motorola Mobility, Inc. All rights reserved.
 * Copyright (C) 2014 Apple Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY MOTOROLA MOBILITY, INC. AND ITS CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL MOTOROLA MOBILITY, INC. OR ITS
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 */

#pragma once

#include "HTMLElement.h"
#include "LiveNodeList.h"
#include <wtf/text/AtomString.h>

namespace WebCore {

class RadioNodeList final : public CachedLiveNodeList<RadioNodeList> {
    WTF_MAKE_ISO_ALLOCATED(RadioNodeList);
public:
    static Ref<RadioNodeList> create(ContainerNode& rootNode, const AtomString& name)
    {
        return adoptRef(*new RadioNodeList(rootNode, name));
    }

    virtual ~RadioNodeList();

    HTMLElement* item(unsigned offset) const override;

    String value() const;
    void setValue(const String&);

    bool elementMatches(Element&) const override;
    bool isRootedAtDocument() const override { return m_isRootedAtDocument; }

private:
    RadioNodeList(ContainerNode&, const AtomString& name);
    bool checkElementMatchesRadioNodeListFilter(const Element&) const;

    AtomString m_name;
    bool m_isRootedAtDocument;
};

inline HTMLElement* RadioNodeList::item(unsigned offset) const
{
    return downcast<HTMLElement>(CachedLiveNodeList<RadioNodeList>::item(offset));
}

} // namepsace WebCore
